@@ -11,6 +11,7 @@ from trytond.pyson import Eval, Bool, If
 from trytond.pool import Pool
 from trytond.transaction import Transaction
 from trytond.wizard import Wizard, StateView, StateTransition, Button
+from trytond.tools import grouped_slice
 
 __all__ = ['Asset', 'AssetLine', 'AssetUpdateMove',
     'CreateMovesStart', 'CreateMoves',
@@ -107,13 +108,15 @@ class Asset(Workflow, ModelSQL, ModelView):
             'readonly': (Eval('lines', [0]) | (Eval('state') != 'draft')),
             },
         required=True,
-        depends=['state'])
+        domain=[('start_date', '<=', Eval('end_date', None))],
+        depends=['state', 'end_date'])
     end_date = fields.Date('End Date',
         states={
             'readonly': (Eval('lines', [0]) | (Eval('state') != 'draft')),
             },
         required=True,
-        depends=['state'])
+        domain=[('end_date', '>=', Eval('start_date', None))],
+        depends=['state', 'start_date'])
     depreciation_method = fields.Selection([
             ('linear', 'Linear'),
             ], 'Depreciation Method',
@@ -302,7 +305,7 @@ class Asset(Workflow, ModelSQL, ModelView):
         # dateutil >= 2.0 has replace __nonzero__ by __bool__ which doesn't
         # work in Python < 3
         if delta == relativedelta.relativedelta():
-            return []
+            return [self.end_date]
         if self.frequency == 'monthly':
             rule = rrule.rrule(rrule.MONTHLY, dtstart=self.start_date,
                 bymonthday=-1)
@@ -326,7 +329,10 @@ class Asset(Workflow, ModelSQL, ModelView):
                     - relativedelta.relativedelta(days=1)]
                 + [l.date for l in self.lines])
             first_delta = dates[0] - start_date
-            last_delta = dates[-1] - dates[-2]
+            if len(dates) > 1:
+                last_delta = dates[-1] - dates[-2]
+            else:
+                last_delta = first_delta
             if self.frequency == 'monthly':
                 _, first_ndays = calendar.monthrange(
                     dates[0].year, dates[0].month)
@@ -360,11 +366,10 @@ class Asset(Workflow, ModelSQL, ModelView):
         asset_line = None
         for date in dates:
             depreciation = self.compute_depreciation(date, dates)
-            with Transaction().set_user(0, set_context=True):
-                amounts[date] = asset_line = Line(
-                    acquired_value=self.value,
-                    depreciable_basis=amount,
-                    )
+            amounts[date] = asset_line = Line(
+                acquired_value=self.value,
+                depreciable_basis=amount,
+                )
             if depreciation > residual_value:
                 asset_line.depreciation = residual_value
                 asset_line.accumulated_depreciation = (
@@ -424,25 +429,24 @@ class Asset(Workflow, ModelSQL, ModelView):
         MoveLine = pool.get('account.move.line')
 
         period_id = Period.find(self.company.id, line.date)
-        with Transaction().set_user(0, set_context=True):
-            expense_line = MoveLine(
-                credit=0,
-                debit=line.depreciation,
-                account=self.product.account_expense_used,
-                )
-            depreciation_line = MoveLine(
-                debit=0,
-                credit=line.depreciation,
-                account=self.product.account_depreciation_used,
-                )
+        expense_line = MoveLine(
+            credit=0,
+            debit=line.depreciation,
+            account=self.product.account_expense_used,
+            )
+        depreciation_line = MoveLine(
+            debit=0,
+            credit=line.depreciation,
+            account=self.product.account_depreciation_used,
+            )
 
-            return Move(
-                origin=line,
-                period=period_id,
-                journal=self.account_journal,
-                date=line.date,
-                lines=[expense_line, depreciation_line],
-                )
+        return Move(
+            origin=line,
+            period=period_id,
+            journal=self.account_journal,
+            date=line.date,
+            lines=[expense_line, depreciation_line],
+            )
 
     @classmethod
     def create_moves(cls, assets, date):
@@ -452,14 +456,12 @@ class Asset(Workflow, ModelSQL, ModelView):
         pool = Pool()
         Move = pool.get('account.move')
         Line = pool.get('account.asset.line')
-        cursor = Transaction().cursor
 
         moves = []
         lines = []
-        for i in range(0, len(assets), cursor.IN_MAX):
-            asset_ids = [a.id for a in assets[i:i + cursor.IN_MAX]]
+        for asset_ids in grouped_slice(assets):
             lines += Line.search([
-                    ('asset', 'in', asset_ids),
+                    ('asset', 'in', list(asset_ids)),
                     ('date', '<=', date),
                     ('move', '=', None),
                     ])
@@ -470,8 +472,7 @@ class Asset(Workflow, ModelSQL, ModelView):
             Line.write([line], {
                     'move': move.id,
                     })
-        with Transaction().set_user(0, set_context=True):
-            Move.post(moves)
+        Move.post(moves)
 
     def get_closing_move(self, account):
         """
@@ -490,37 +491,34 @@ class Asset(Workflow, ModelSQL, ModelView):
         else:
             account_asset = self.product.account_asset_used
 
-        with Transaction().set_user(0, set_context=True):
-            asset_line = MoveLine(
-                debit=0,
-                credit=self.value,
-                account=account_asset,
-                )
-            depreciation_line = MoveLine(
-                debit=self.get_depreciated_amount(),
-                credit=0,
-                account=self.product.account_depreciation_used,
-                )
+        asset_line = MoveLine(
+            debit=0,
+            credit=self.value,
+            account=account_asset,
+            )
+        depreciation_line = MoveLine(
+            debit=self.get_depreciated_amount(),
+            credit=0,
+            account=self.product.account_depreciation_used,
+            )
         lines = [asset_line, depreciation_line]
         square_amount = asset_line.credit - depreciation_line.debit
         if square_amount:
             if not account:
                 account = self.product.account_revenue_used
-            with Transaction().set_user(0, set_context=True):
-                counter_part_line = MoveLine(
-                    debit=square_amount if square_amount > 0 else 0,
-                    credit=-square_amount if square_amount < 0 else 0,
-                    account=account,
-                    )
-            lines.append(counter_part_line)
-        with Transaction().set_user(0, set_context=True):
-            return Move(
-                origin=self,
-                period=period_id,
-                journal=self.account_journal,
-                date=date,
-                lines=lines,
+            counter_part_line = MoveLine(
+                debit=square_amount if square_amount > 0 else 0,
+                credit=-square_amount if square_amount < 0 else 0,
+                account=account,
                 )
+            lines.append(counter_part_line)
+        return Move(
+            origin=self,
+            period=period_id,
+            journal=self.account_journal,
+            date=date,
+            lines=lines,
+            )
 
     @classmethod
     def set_reference(cls, assets):
@@ -566,8 +564,7 @@ class Asset(Workflow, ModelSQL, ModelView):
             cls.write([asset], {
                     'move': move.id,
                     })
-        with Transaction().set_user(0, set_context=True):
-            Move.post(moves)
+        Move.post(moves)
 
     def get_rec_name(self, name):
         return '%s - %s' % (self.reference, self.product.rec_name)
@@ -670,7 +667,17 @@ class UpdateAssetShowDepreciation(ModelView):
     'Update Asset Show Depreciation'
     __name__ = 'account.asset.update.show_depreciation'
     amount = fields.Numeric('Amount', readonly=True)
-    date = fields.Date('Date', required=True)
+    date = fields.Date('Date', required=True,
+        domain=[
+            ('date', '>', Eval('latest_move_date')),
+            ('date', '<', Eval('next_depreciation_date')),
+            ],
+        depends=['latest_move_date', 'next_depreciation_date'],
+        help=('The date must be between the last update/depreciation date '
+            'and the next depreciation date.'))
+    latest_move_date = fields.Date('Latest Move Date', readonly=True)
+    next_depreciation_date = fields.Date('Next Depreciation Date',
+        readonly=True)
     depreciation_account = fields.Many2One('account.account',
         'Depreciation Account', readonly=True)
     counterpart_account = fields.Many2One('account.account',
@@ -710,6 +717,21 @@ class UpdateAsset(Wizard):
             return 'show_move'
         return 'create_lines'
 
+    def get_latest_move_date(self, asset):
+        previous_dates = [datetime.date.min]
+        previous_dates += [m.date for m in asset.update_moves
+            if m.state == 'posted']
+        previous_dates += [l.date for l in asset.lines
+                if l.move and l.move.state == 'posted']
+        return max(previous_dates)
+
+    def get_next_depreciation_date(self, asset):
+        next_dates = [datetime.date.max]
+        next_dates += [l.date for l in asset.lines
+            if not l.move or l.move.state != 'posted']
+
+        return min(next_dates)
+
     def default_show_move(self, fields):
         Asset = Pool().get('account.asset')
         asset = Asset(Transaction().context['active_id'])
@@ -718,6 +740,8 @@ class UpdateAsset(Wizard):
             'date': datetime.date.today(),
             'depreciation_account': asset.product.account_depreciation_used.id,
             'counterpart_account': asset.product.account_expense_used.id,
+            'latest_move_date': self.get_latest_move_date(asset),
+            'next_depreciation_date': self.get_next_depreciation_date(asset),
             }
 
     def get_move(self, asset):
@@ -752,6 +776,10 @@ class UpdateAsset(Wizard):
         Asset = pool.get('account.asset')
 
         asset = Asset(Transaction().context['active_id'])
+        latest_move_date = self.show_move.latest_move_date
+        next_date = self.show_move.next_depreciation_date
+        if not (latest_move_date < self.show_move.date < next_date):
+            raise ValueError('The update move date is invalid')
         move = self.get_move(asset)
         move.lines = self.get_move_lines(asset)
         move.save()
